@@ -1,4 +1,4 @@
-﻿#-------------------------------------------------------------------------
+# -------------------------------------------------------------------------
 # Copyright (c) Microsoft.  All rights reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -11,7 +11,8 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-#--------------------------------------------------------------------------
+# --------------------------------------------------------------------------
+from .fileservice import FileService
 from .models import (
     Share,
     ShareProperties,
@@ -24,6 +25,5 @@ from .models import (
     CopyProperties,
     SharePermissions,
     FilePermissions,
+    DeleteSnapshot,
 )
-
-from .fileservice import FileService
