@@ -4,16 +4,13 @@
 # license information.
 # --------------------------------------------------------------------------
 # pylint: disable=no-self-use
-
+from azure.core.exceptions import HttpResponseError
 from .._deserialize import (
     process_storage_error)
-from .._generated.models import (
-    StorageErrorException,
-)
 from .._shared.response_handlers import return_response_headers
 from .._shared.uploads_async import (
     upload_data_chunks,
-    DataLakeFileChunkUploader)
+    DataLakeFileChunkUploader, upload_substream_blocks)
 
 
 def _any_conditions(modified_access_conditions=None, **kwargs):  # pylint: disable=unused-argument
@@ -32,6 +29,7 @@ async def upload_datalake_file(  # pylint: disable=unused-argument
         overwrite=None,
         validate_content=None,
         max_concurrency=None,
+        file_settings=None,
         **kwargs):
     try:
         if length == 0:
@@ -68,20 +66,38 @@ async def upload_datalake_file(  # pylint: disable=unused-argument
             modified_access_conditions.if_modified_since = None
             modified_access_conditions.if_unmodified_since = None
 
-        await upload_data_chunks(
-            service=client,
-            uploader_class=DataLakeFileChunkUploader,
-            total_size=length,
-            chunk_size=chunk_size,
-            stream=stream,
-            max_concurrency=max_concurrency,
-            validate_content=validate_content,
-            **kwargs)
+        use_original_upload_path = file_settings.use_byte_buffer or \
+            validate_content or chunk_size < file_settings.min_large_chunk_upload_threshold or \
+            hasattr(stream, 'seekable') and not stream.seekable() or \
+            not hasattr(stream, 'seek') or not hasattr(stream, 'tell')
+
+        if use_original_upload_path:
+            await upload_data_chunks(
+                service=client,
+                uploader_class=DataLakeFileChunkUploader,
+                total_size=length,
+                chunk_size=chunk_size,
+                stream=stream,
+                max_concurrency=max_concurrency,
+                validate_content=validate_content,
+                **kwargs)
+        else:
+            await upload_substream_blocks(
+                service=client,
+                uploader_class=DataLakeFileChunkUploader,
+                total_size=length,
+                chunk_size=chunk_size,
+                max_concurrency=max_concurrency,
+                stream=stream,
+                validate_content=validate_content,
+                **kwargs
+            )
 
         return await client.flush_data(position=length,
                                        path_http_headers=path_http_headers,
                                        modified_access_conditions=modified_access_conditions,
+                                       close=True,
                                        cls=return_response_headers,
                                        **kwargs)
-    except StorageErrorException as error:
+    except HttpResponseError as error:
         process_storage_error(error)
