@@ -9,20 +9,23 @@ from typing import (  # pylint: disable=unused-import
     Union, Optional, Any, Dict, List,
     TYPE_CHECKING
 )
+
+
 try:
     from urllib.parse import urlparse
 except ImportError:
     from urlparse import urlparse # type: ignore
 
+from azure.core.exceptions import HttpResponseError
 from azure.core.paging import ItemPaged
 from azure.core.tracing.decorator import distributed_trace
 from azure.core.pipeline import Pipeline
 from ._shared.base_client import StorageAccountHostsMixin, TransportWrapper, parse_connection_str, parse_query
 from ._shared.response_handlers import process_storage_error
 from ._generated import AzureFileStorage
-from ._generated.models import StorageErrorException, StorageServiceProperties
-from ._generated.version import VERSION
+from ._generated.models import StorageServiceProperties
 from ._share_client import ShareClient
+from ._serialize import get_api_version
 from ._models import (
     SharePropertiesPaged,
     service_properties_deserialize,
@@ -34,6 +37,7 @@ if TYPE_CHECKING:
         ShareProperties,
         Metrics,
         CorsRule,
+        ShareProtocolSettings
     )
 
 
@@ -45,14 +49,29 @@ class ShareServiceClient(StorageAccountHostsMixin):
     For operations relating to a specific share, a client for that entity
     can also be retrieved using the :func:`get_share_client` function.
 
+    For more optional configuration, please click
+    `here <https://github.com/Azure/azure-sdk-for-python/tree/main/sdk/storage/azure-storage-file-share
+    #optional-configuration>`_.
+
     :param str account_url:
         The URL to the file share storage account. Any other entities included
         in the URL path (e.g. share or file) will be discarded. This URL can be optionally
         authenticated with a SAS token.
     :param credential:
-        The credential with which to authenticate. This is optional if the
-        account URL already has a SAS token. The value can be a SAS token string or an account
-        shared access key.
+        The credentials with which to authenticate. This is optional if the
+        account URL already has a SAS token. The value can be a SAS token string,
+        an instance of a AzureSasCredential or AzureNamedKeyCredential from azure.core.credentials,
+        an account shared access key, or an instance of a TokenCredentials class from azure.identity.
+        If the resource URI already contains a SAS token, this will be ignored in favor of an explicit credential
+        - except in the case of AzureSasCredential, where the conflicting SAS tokens will raise a ValueError.
+        If using an instance of AzureNamedKeyCredential, "name" should be the storage account name, and "key"
+        should be the storage account key.
+    :keyword str api_version:
+        The Storage API version to use for requests. Default value is the most recent service version that is
+        compatible with the current SDK. Setting to an older version may result in reduced feature compatibility.
+
+        .. versionadded:: 12.1.0
+
     :keyword str secondary_hostname:
         The hostname of the secondary endpoint.
     :keyword int max_range_size: The maximum range size used for a file upload. Defaults to 4*1024*1024.
@@ -68,7 +87,7 @@ class ShareServiceClient(StorageAccountHostsMixin):
     """
     def __init__(
             self, account_url,  # type: str
-            credential=None,  # type: Optional[Any]
+            credential=None,  # type: Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, "TokenCredential"]] # pylint: disable=line-too-long
             **kwargs  # type: Any
         ):
         # type: (...) -> None
@@ -89,7 +108,8 @@ class ShareServiceClient(StorageAccountHostsMixin):
                 'You need to provide either an account shared key or SAS token when creating a storage service.')
         self._query_str, credential = self._format_query_string(sas_token, credential)
         super(ShareServiceClient, self).__init__(parsed_url, service='file-share', credential=credential, **kwargs)
-        self._client = AzureFileStorage(version=VERSION, url=self.url, pipeline=self._pipeline)
+        self._client = AzureFileStorage(url=self.url, base_url=self.url, pipeline=self._pipeline)
+        self._client._config.version = get_api_version(kwargs) # pylint: disable=protected-access
 
     def _format_url(self, hostname):
         """Format the endpoint URL according to the current location
@@ -100,7 +120,7 @@ class ShareServiceClient(StorageAccountHostsMixin):
     @classmethod
     def from_connection_string(
             cls, conn_str,  # type: str
-            credential=None, # type: Optional[Any]
+            credential=None,  # type: Optional[Union[str, Dict[str, str], AzureNamedKeyCredential, AzureSasCredential, "TokenCredential"]] # pylint: disable=line-too-long
             **kwargs  # type: Any
         ):  # type: (...) -> ShareServiceClient
         """Create ShareServiceClient from a Connection String.
@@ -108,9 +128,14 @@ class ShareServiceClient(StorageAccountHostsMixin):
         :param str conn_str:
             A connection string to an Azure Storage account.
         :param credential:
-            The credential with which to authenticate. This is optional if the
-            account URL already has a SAS token. The value can be a SAS token string or an account
-            shared access key.
+            The credentials with which to authenticate. This is optional if the
+            account URL already has a SAS token. The value can be a SAS token string,
+            an instance of a AzureSasCredential or AzureNamedKeyCredential from azure.core.credentials,
+            an account shared access key, or an instance of a TokenCredentials class from azure.identity.
+            If the resource URI already contains a SAS token, this will be ignored in favor of an explicit credential
+            - except in the case of AzureSasCredential, where the conflicting SAS tokens will raise a ValueError.
+            If using an instance of AzureNamedKeyCredential, "name" should be the storage account name, and "key"
+            should be the storage account key.
         :returns: A File Share service client.
         :rtype: ~azure.storage.fileshare.ShareServiceClient
 
@@ -153,7 +178,7 @@ class ShareServiceClient(StorageAccountHostsMixin):
         try:
             service_props = self._client.service.get_properties(timeout=timeout, **kwargs)
             return service_properties_deserialize(service_props)
-        except StorageErrorException as error:
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -161,6 +186,7 @@ class ShareServiceClient(StorageAccountHostsMixin):
             self, hour_metrics=None,  # type: Optional[Metrics]
             minute_metrics=None,  # type: Optional[Metrics]
             cors=None,  # type: Optional[List[CorsRule]]
+            protocol=None,  # type: Optional[ShareProtocolSettings],
             **kwargs
         ):
         # type: (...) -> None
@@ -181,6 +207,9 @@ class ShareServiceClient(StorageAccountHostsMixin):
             list. If an empty list is specified, all CORS rules will be deleted,
             and CORS will be disabled for the service.
         :type cors: list(:class:`~azure.storage.fileshare.CorsRule`)
+        :param protocol:
+            Sets protocol settings
+        :type protocol: ~azure.storage.fileshare.ShareProtocolSettings
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :rtype: None
@@ -198,11 +227,12 @@ class ShareServiceClient(StorageAccountHostsMixin):
         props = StorageServiceProperties(
             hour_metrics=hour_metrics,
             minute_metrics=minute_metrics,
-            cors=cors
+            cors=cors,
+            protocol=protocol
         )
         try:
-            self._client.service.set_properties(props, timeout=timeout, **kwargs)
-        except StorageErrorException as error:
+            self._client.service.set_properties(storage_service_properties=props, timeout=timeout, **kwargs)
+        except HttpResponseError as error:
             process_storage_error(error)
 
     @distributed_trace
@@ -224,6 +254,9 @@ class ShareServiceClient(StorageAccountHostsMixin):
             Specifies that share metadata be returned in the response.
         :param bool include_snapshots:
             Specifies that share snapshot be returned in the response.
+        :keyword bool include_deleted:
+            Specifies that deleted shares be returned in the response.
+            This is only for share soft delete enabled account.
         :keyword int timeout:
             The timeout parameter is expressed in seconds.
         :returns: An iterable (auto-paging) of ShareProperties.
@@ -240,10 +273,14 @@ class ShareServiceClient(StorageAccountHostsMixin):
         """
         timeout = kwargs.pop('timeout', None)
         include = []
+        include_deleted = kwargs.pop('include_deleted', None)
+        if include_deleted:
+            include.append("deleted")
         if include_metadata:
             include.append('metadata')
         if include_snapshots:
             include.append('snapshots')
+
         results_per_page = kwargs.pop('results_per_page', None)
         command = functools.partial(
             self._client.service.list_shares_segment,
@@ -326,6 +363,35 @@ class ShareServiceClient(StorageAccountHostsMixin):
         share.delete_share(
             delete_snapshots=delete_snapshots, timeout=timeout, **kwargs)
 
+    @distributed_trace
+    def undelete_share(self, deleted_share_name, deleted_share_version, **kwargs):
+        # type: (str, str, **Any) -> ShareClient
+        """Restores soft-deleted share.
+
+        Operation will only be successful if used within the specified number of days
+        set in the delete retention policy.
+
+        .. versionadded:: 12.2.0
+            This operation was introduced in API version '2019-12-12'.
+
+        :param str deleted_share_name:
+            Specifies the name of the deleted share to restore.
+        :param str deleted_share_version:
+            Specifies the version of the deleted share to restore.
+        :keyword int timeout:
+            The timeout parameter is expressed in seconds.
+        :rtype: ~azure.storage.fileshare.ShareClient
+        """
+        share = self.get_share_client(deleted_share_name)
+
+        try:
+            share._client.share.restore(deleted_share_name=deleted_share_name,  # pylint: disable = protected-access
+                                        deleted_share_version=deleted_share_version,
+                                        timeout=kwargs.pop('timeout', None), **kwargs)
+            return share
+        except HttpResponseError as error:
+            process_storage_error(error)
+
     def get_share_client(self, share, snapshot=None):
         # type: (Union[ShareProperties, str],Optional[Union[Dict[str, Any], str]]) -> ShareClient
         """Get a client to interact with the specified share.
@@ -360,5 +426,6 @@ class ShareServiceClient(StorageAccountHostsMixin):
             policies=self._pipeline._impl_policies # pylint: disable = protected-access
         )
         return ShareClient(
-            self.url, share_name=share_name, snapshot=snapshot, credential=self.credential, _hosts=self._hosts,
+            self.url, share_name=share_name, snapshot=snapshot, credential=self.credential,
+            api_version=self.api_version, _hosts=self._hosts,
             _configuration=self._config, _pipeline=_pipeline, _location_mode=self._location_mode)
